@@ -130,6 +130,11 @@ public:
                 r += std::to_string(down_cast<ASR::Real_t>(t)->m_kind);
                 r += ")";
                 break;
+	    } case ASR::ttypeType::Complex: {
+                r = "complex(";
+                r += std::to_string(down_cast<ASR::Complex_t>(t)->m_kind);
+                r += ")";
+                break;
             } case ASR::ttypeType::Character: {
                 ASR::Character_t *c = down_cast<ASR::Character_t>(t);
                 r = "character(len=";
@@ -580,11 +585,102 @@ public:
         s = r;
     }
 
-    // void visit_FileOpen(const ASR::FileOpen_t &x) {}
+    void visit_FileOpen(const ASR::FileOpen_t &x) {
+        std::string r;
+        r = indent;
+        r += "open";
+        r += "(";
+        if (x.m_newunit) {
+            visit_expr(*x.m_newunit);
+            r += s;
+        } else {
+            throw CodeGenError("open() function must be called with a file unit number");
+        }
+        if (x.m_filename) {
+            r += ", ";
+            r += "file=";
+            visit_expr(*x.m_filename);
+            r += s;
+        }
+        if (x.m_status) {
+            r += ", ";
+            r += "status=";
+            visit_expr(*x.m_status);
+            r += s;
+        }
+        if (x.m_form) {
+            r += ", ";
+            r += "form=";
+            visit_expr(*x.m_form);
+            r += s;
+        }
+        r += ")";
+        r += "\n";
+        s = r;
+    }
 
-    // void visit_FileClose(const ASR::FileClose_t &x) {}
+    void visit_FileClose(const ASR::FileClose_t &x) {
+        std::string r;
+        r = indent;
+        r += "close";
+        r += "(";
+        if (x.m_unit) {
+            visit_expr(*x.m_unit);
+            r += s;
+        } else {
+            throw CodeGenError("close() function must be called with a file unit number");
+        }
+        r += ")";
+        r += "\n";
+        s = r;
+    }
 
-    // void visit_FileRead(const ASR::FileRead_t &x) {}
+   void visit_FileRead(const ASR::FileRead_t &x) {
+        std::string r;
+        r = indent;
+        r += "read";
+        r += "(";
+        if (x.m_unit) {
+            visit_expr(*x.m_unit);
+            r += s;
+        } else {
+            r += "*";
+        }
+        if (x.m_fmt) {
+            r += ", ";
+            r += "fmt=";
+            visit_expr(*x.m_fmt);
+            r += s;
+        } else {
+            r += ", *";
+        }
+        if (x.m_iomsg) {
+            r += ", ";
+            r += "iomsg=";
+            visit_expr(*x.m_iomsg);
+            r += s;
+        }
+        if (x.m_iostat) {
+            r += ", ";
+            r += "iostat=";
+            visit_expr(*x.m_iostat);
+            r += s;
+        }
+        if (x.m_id) {
+            r += ", ";
+            r += "id=";
+            visit_expr(*x.m_id);
+            r += s;
+        }
+        r += ") ";
+        for (size_t i = 0; i < x.n_values; i++) {
+            visit_expr(*x.m_values[i]);
+            r += s;
+            if (i < x.n_values - 1) r += ", ";
+        }
+        r += "\n";
+        s = r;
+    }
 
     // void visit_FileBackspace(const ASR::FileBackspace_t &x) {}
 
@@ -743,7 +839,13 @@ public:
     /********************************** Expr **********************************/
     // void visit_IfExp(const ASR::IfExp_t &x) {}
 
-    // void visit_ComplexConstructor(const ASR::ComplexConstructor_t &x) {}
+    void visit_ComplexConstructor(const ASR::ComplexConstructor_t &x) {
+        visit_expr(*x.m_re);
+        std::string re = s;
+        visit_expr(*x.m_im);
+        std::string im = s;
+        s = "(" + re + ", " + im + ")";
+    }
 
     // void visit_NamedExpr(const ASR::NamedExpr_t &x) {}
 
@@ -764,6 +866,8 @@ public:
         switch (x.m_intrinsic_id) {
             SET_INTRINSIC_NAME(Abs, "abs");
             SET_INTRINSIC_NAME(Exp, "exp");
+            SET_INTRINSIC_NAME(Max, "max");
+            SET_INTRINSIC_NAME(Min, "min");
             default : {
                 throw LCompilersException("IntrinsicScalarFunction: `"
                     + ASRUtils::get_intrinsic_name(x.m_intrinsic_id)
@@ -785,6 +889,7 @@ public:
     void visit_IntrinsicArrayFunction(const ASR::IntrinsicArrayFunction_t &x) {
         std::string out;
         switch (x.m_arr_intrinsic_id) {
+            SET_ARR_INTRINSIC_NAME(Any, "any");
             SET_ARR_INTRINSIC_NAME(Sum, "sum");
             SET_ARR_INTRINSIC_NAME(Shape, "shape");
             default : {
@@ -902,7 +1007,11 @@ public:
 
     // void visit_RealCopySign(const ASR::RealCopySign_t &x) {}
 
-    // void visit_ComplexConstant(const ASR::ComplexConstant_t &x) {}
+    void visit_ComplexConstant(const ASR::ComplexConstant_t &x) {
+        std::string re = std::to_string(x.m_re);
+        std::string im = std::to_string(x.m_im);
+        s = "(" + re + ", " + im + ")";
+    }
 
     // void visit_ComplexUnaryMinus(const ASR::ComplexUnaryMinus_t &x) {}
 
@@ -956,7 +1065,13 @@ public:
         s = left + "//" + right;
     }
 
-    // void visit_StringRepeat(const ASR::StringRepeat_t &x) {}
+    void visit_StringRepeat(const ASR::StringRepeat_t &x) {
+        this->visit_expr(*x.m_left);
+        std::string str = s;
+        this->visit_expr(*x.m_right);
+        std::string n = s;
+        s = "repeat(" + str + ", " + n + ")";
+    }
 
     void visit_StringLen(const ASR::StringLen_t &x) {
         visit_expr(*x.m_arg);
@@ -1312,9 +1427,15 @@ public:
         this->visit_expr(*x.m_arg);
     }
 
-    // void visit_ComplexRe(const ASR::ComplexRe_t &x) {}
+    void visit_ComplexRe(const ASR::ComplexRe_t &x) {
+        visit_expr(*x.m_arg);
+        s = "real(" + s + ")";
+    }
 
-    // void visit_ComplexIm(const ASR::ComplexIm_t &x) {}
+    void visit_ComplexIm(const ASR::ComplexIm_t &x) {
+        visit_expr(*x.m_arg);
+        s = "aimag(" + s + ")";
+    }
 
     // void visit_CLoc(const ASR::CLoc_t &x) {}
 
@@ -1322,11 +1443,20 @@ public:
 
     // void visit_GetPointer(const ASR::GetPointer_t &x) {}
 
-    // void visit_IntegerBitLen(const ASR::IntegerBitLen_t &x) {}
+    void visit_IntegerBitLen(const ASR::IntegerBitLen_t &x) {
+        visit_expr(*x.m_a);
+        s = "bit_size(" + s + ")";
+    }
 
-    // void visit_Ichar(const ASR::Ichar_t &x) {}
+    void visit_Ichar(const ASR::Ichar_t &x) {
+        visit_expr(*x.m_arg);
+        s = "ichar(" + s + ")";
+    }
 
-    // void visit_Iachar(const ASR::Iachar_t &x) {}
+    void visit_Iachar(const ASR::Iachar_t &x) {
+        visit_expr(*x.m_arg);
+        s = "iachar(" + s + ")";
+    }
 
     // void visit_SizeOfType(const ASR::SizeOfType_t &x) {}
 
@@ -1334,7 +1464,10 @@ public:
 
     // void visit_PointerAssociated(const ASR::PointerAssociated_t &x) {}
 
-    // void visit_IntrinsicFunctionSqrt(const ASR::IntrinsicFunctionSqrt_t &x) {}
+    void visit_IntrinsicFunctionSqrt(const ASR::IntrinsicFunctionSqrt_t &x) {
+        visit_expr(*x.m_arg);
+        s = "sqrt(" + s + ")";
+    }
 
     /******************************* Case Stmt ********************************/
     void visit_CaseStmt(const ASR::CaseStmt_t &x) {
